@@ -18,6 +18,8 @@ use crate::x86::Ring;
 use alloc::boxed::Box;
 use core::mem;
 
+use super::scheduler::TaskStacks;
+
 pub const GDT_NULL: u16 = 0;
 pub const GDT_KERNEL_CODE: u16 = 1;
 pub const GDT_KERNEL_DATA: u16 = 2;
@@ -95,14 +97,14 @@ pub fn add_current_core() {
 	// When switching to another task on this core, this entry is replaced.
 	boxed_tss.rsp[0] = unsafe { core::ptr::read_volatile(&(*BOOT_INFO).current_stack_address) }
 		+ KERNEL_STACK_SIZE as u64
-		- 0x10u64;
+		- TaskStacks::MARKER_SIZE as u64;
 	set_kernel_stack(boxed_tss.rsp[0] as u64);
 
 	// Allocate all ISTs for this core.
 	// Every task later gets its own IST1, so the IST1 allocated here is only used by the Idle task.
 	for i in 0..IST_ENTRIES {
 		let ist = crate::mm::allocate(KERNEL_STACK_SIZE, true);
-		boxed_tss.ist[i] = ist.as_u64() + KERNEL_STACK_SIZE as u64 - 0x10u64;
+		boxed_tss.ist[i] = ist.as_u64() + KERNEL_STACK_SIZE as u64 - TaskStacks::MARKER_SIZE as u64;
 	}
 
 	unsafe {
